# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meshtastic/protobuf/interdevice.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%meshtastic/protobuf/interdevice.proto\x12\x13meshtastic.protobuf\"s\n\nSensorData\x12.\n\x04type\x18\x01 \x01(\x0e\x32 .meshtastic.protobuf.MessageType\x12\x15\n\x0b\x66loat_value\x18\x02 \x01(\x02H\x00\x12\x16\n\x0cuint32_value\x18\x03 \x01(\rH\x00\x42\x06\n\x04\x64\x61ta\"_\n\x12InterdeviceMessage\x12\x0e\n\x04nmea\x18\x01 \x01(\tH\x00\x12\x31\n\x06sensor\x18\x02 \x01(\x0b\x32\x1f.meshtastic.protobuf.SensorDataH\x00\x42\x06\n\x04\x64\x61ta*\xd5\x01\n\x0bMessageType\x12\x07\n\x03\x41\x43K\x10\x00\x12\x15\n\x10\x43OLLECT_INTERVAL\x10\xa0\x01\x12\x0c\n\x07\x42\x45\x45P_ON\x10\xa1\x01\x12\r\n\x08\x42\x45\x45P_OFF\x10\xa2\x01\x12\r\n\x08SHUTDOWN\x10\xa3\x01\x12\r\n\x08POWER_ON\x10\xa4\x01\x12\x0f\n\nSCD41_TEMP\x10\xb0\x01\x12\x13\n\x0eSCD41_HUMIDITY\x10\xb1\x01\x12\x0e\n\tSCD41_CO2\x10\xb2\x01\x12\x0f\n\nAHT20_TEMP\x10\xb3\x01\x12\x13\n\x0e\x41HT20_HUMIDITY\x10\xb4\x01\x12\x0f\n\nTVOC_INDEX\x10\xb5\x01\x42g\n\x14org.meshtastic.protoB\x11InterdeviceProtosZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtastic.protobuf.interdevice_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024org.meshtastic.protoB\021InterdeviceProtosZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _globals['_MESSAGETYPE']._serialized_start=277
  _globals['_MESSAGETYPE']._serialized_end=490
  _globals['_SENSORDATA']._serialized_start=62
  _globals['_SENSORDATA']._serialized_end=177
  _globals['_INTERDEVICEMESSAGE']._serialized_start=179
  _globals['_INTERDEVICEMESSAGE']._serialized_end=274
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _MessageType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MessageTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MessageType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ACK: _MessageType.ValueType  # 0
    COLLECT_INTERVAL: _MessageType.ValueType  # 160
    """in ms"""
    BEEP_ON: _MessageType.ValueType  # 161
    """duration ms"""
    BEEP_OFF: _MessageType.ValueType  # 162
    """cancel prematurely"""
    SHUTDOWN: _MessageType.ValueType  # 163
    POWER_ON: _MessageType.ValueType  # 164
    SCD41_TEMP: _MessageType.ValueType  # 176
    SCD41_HUMIDITY: _MessageType.ValueType  # 177
    SCD41_CO2: _MessageType.ValueType  # 178
    AHT20_TEMP: _MessageType.ValueType  # 179
    AHT20_HUMIDITY: _MessageType.ValueType  # 180
    TVOC_INDEX: _MessageType.ValueType  # 181

class MessageType(_MessageType, metaclass=_MessageTypeEnumTypeWrapper):
    """encapsulate up to 1k of NMEA string data"""

ACK: MessageType.ValueType  # 0
COLLECT_INTERVAL: MessageType.ValueType  # 160
"""in ms"""
BEEP_ON: MessageType.ValueType  # 161
"""duration ms"""
BEEP_OFF: MessageType.ValueType  # 162
"""cancel prematurely"""
SHUTDOWN: MessageType.ValueType  # 163
POWER_ON: MessageType.ValueType  # 164
SCD41_TEMP: MessageType.ValueType  # 176
SCD41_HUMIDITY: MessageType.ValueType  # 177
SCD41_CO2: MessageType.ValueType  # 178
AHT20_TEMP: MessageType.ValueType  # 179
AHT20_HUMIDITY: MessageType.ValueType  # 180
TVOC_INDEX: MessageType.ValueType  # 181
global___MessageType = MessageType

@typing.final
class SensorData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    FLOAT_VALUE_FIELD_NUMBER: builtins.int
    UINT32_VALUE_FIELD_NUMBER: builtins.int
    type: global___MessageType.ValueType
    """The message type"""
    float_value: builtins.float
    uint32_value: builtins.int
    def __init__(
        self,
        *,
        type: global___MessageType.ValueType = ...,
        float_value: builtins.float = ...,
        uint32_value: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data", "float_value", b"float_value", "uint32_value", b"uint32_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "float_value", b"float_value", "type", b"type", "uint32_value", b"uint32_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["data", b"data"]) -> typing.Literal["float_value", "uint32_value"] | None: ...

global___SensorData = SensorData

@typing.final
class InterdeviceMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NMEA_FIELD_NUMBER: builtins.int
    SENSOR_FIELD_NUMBER: builtins.int
    nmea: builtins.str
    @property
    def sensor(self) -> global___SensorData: ...
    def __init__(
        self,
        *,
        nmea: builtins.str = ...,
        sensor: global___SensorData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data", "nmea", b"nmea", "sensor", b"sensor"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "nmea", b"nmea", "sensor", b"sensor"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["data", b"data"]) -> typing.Literal["nmea", "sensor"] | None: ...

global___InterdeviceMessage = InterdeviceMessage



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meshtastic/protobuf/paxcount.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"meshtastic/protobuf/paxcount.proto\x12\x13meshtastic.protobuf\"5\n\x08Paxcount\x12\x0c\n\x04wifi\x18\x01 \x01(\r\x12\x0b\n\x03\x62le\x18\x02 \x01(\r\x12\x0e\n\x06uptime\x18\x03 \x01(\rBd\n\x14org.meshtastic.protoB\x0ePaxcountProtosZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtastic.protobuf.paxcount_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024org.meshtastic.protoB\016PaxcountProtosZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _globals['_PAXCOUNT']._serialized_start=59
  _globals['_PAXCOUNT']._serialized_end=112
# @@protoc_insertion_point(module_scope)



"""Meshtastic unit tests for showNodes favorite column feature"""

from unittest.mock import MagicMock

import pytest

from ..mesh_interface import MeshInterface


@pytest.fixture
def _iface_with_favorite_nodes():
    """Fixture to setup nodes with favorite flags."""
    nodesById = {
        "!9388f81c": {
            "num": 2475227164,
            "user": {
                "id": "!9388f81c",
                "longName": "Favorite Node",
                "shortName": "FAV1",
                "macaddr": "RBeTiPgc",
                "hwModel": "TBEAM",
            },
            "position": {},
            "lastHeard": 1640204888,
            "isFavorite": True,
        },
        "!12345678": {
            "num": 305419896,
            "user": {
                "id": "!12345678",
                "longName": "Regular Node",
                "shortName": "REG1",
                "macaddr": "ABCDEFGH",
                "hwModel": "TLORA_V2",
            },
            "position": {},
            "lastHeard": 1640204999,
            "isFavorite": False,
        },
        "!abcdef00": {
            "num": 2882400000,
            "user": {
                "id": "!abcdef00",
                "longName": "Legacy Node",
                "shortName": "LEG1",
                "macaddr": "XYZABC00",
                "hwModel": "HELTEC_V3",
            },
            "position": {},
            "lastHeard": 1640205000,
            # Note: No isFavorite field - testing backward compatibility
        },
    }

    nodesByNum = {
        2475227164: {
            "num": 2475227164,
            "user": {
                "id": "!9388f81c",
                "longName": "Favorite Node",
                "shortName": "FAV1",
                "macaddr": "RBeTiPgc",
                "hwModel": "TBEAM",
            },
            "position": {"time": 1640206266},
            "lastHeard": 1640206266,
            "isFavorite": True,
        },
        305419896: {
            "num": 305419896,
            "user": {
                "id": "!12345678",
                "longName": "Regular Node",
                "shortName": "REG1",
                "macaddr": "ABCDEFGH",
                "hwModel": "TLORA_V2",
            },
            "position": {"time": 1640206200},
            "lastHeard": 1640206200,
            "isFavorite": False,
        },
        2882400000: {
            "num": 2882400000,
            "user": {
                "id": "!abcdef00",
                "longName": "Legacy Node",
                "shortName": "LEG1",
                "macaddr": "XYZABC00",
                "hwModel": "HELTEC_V3",
            },
            "position": {"time": 1640206100},
            "lastHeard": 1640206100,
            # Note: No isFavorite field - testing backward compatibility
        },
    }

    iface = MeshInterface(noProto=True)
    iface.nodes = nodesById
    iface.nodesByNum = nodesByNum
    myInfo = MagicMock()
    iface.myInfo = myInfo
    iface.myInfo.my_node_num = 2475227164
    return iface


@pytest.mark.unit
def test_showNodes_favorite_column_header(capsys, _iface_with_favorite_nodes):
    """Test that 'Fav' column header appears in showNodes output"""
    iface = _iface_with_favorite_nodes
    iface.showNodes()
    out, err = capsys.readouterr()
    assert "Fav" in out
    assert err == ""


@pytest.mark.unit
def test_showNodes_favorite_asterisk_display(capsys, _iface_with_favorite_nodes):
    """Test that favorite nodes show asterisk and non-favorites show empty"""
    iface = _iface_with_favorite_nodes
    iface.showNodes()
    out, err = capsys.readouterr()

    # Check that the output contains the "Fav" column
    assert "Fav" in out

    # Find lines containing our nodes
    lines = out.split('\n')
    favorite_line = None
    regular_line = None
    legacy_line = None
    for line in lines:
        if "Favorite Node" in line or "FAV1" in line:
            favorite_line = line
        if "Regular Node" in line or "REG1" in line:
            regular_line = line
        if "Legacy Node" in line or "LEG1" in line:
            legacy_line = line

    # Verify all nodes are present in the output
    assert favorite_line is not None, "Favorite node should be in output"
    assert regular_line is not None, "Regular node should be in output"
    assert legacy_line is not None, "Legacy node should be in output"

    # Verify the favorite node has an asterisk in its row
    assert "*" in favorite_line, "Favorite node should have an asterisk"

    # Verify the regular (non-favorite) node does NOT have an asterisk
    assert regular_line.count("*") == 0, "Non-favorite node should not have an asterisk"

    # Verify the legacy node (without isFavorite field) does NOT have an asterisk
    assert legacy_line.count("*") == 0, "Legacy node without isFavorite field should not have an asterisk"

    assert err == ""


@pytest.mark.unit
def test_showNodes_favorite_field_formatting():
    """Test the formatting logic for isFavorite field"""
    # Test favorite node
    raw_value = True
    formatted_value = "*" if raw_value else ""
    assert formatted_value == "*"

    # Test non-favorite node
    raw_value = False
    formatted_value = "*" if raw_value else ""
    assert formatted_value == ""

    # Test None/missing value
    raw_value = None
    formatted_value = "*" if raw_value else ""
    assert formatted_value == ""


@pytest.mark.unit
def test_showNodes_with_custom_fields_including_favorite(capsys, _iface_with_favorite_nodes):
    """Test that isFavorite can be specified in custom showFields"""
    iface = _iface_with_favorite_nodes
    custom_fields = ["user.longName", "isFavorite"]
    iface.showNodes(showFields=custom_fields)
    out, err = capsys.readouterr()

    # Should still show the Fav column when explicitly requested
    assert "Fav" in out
    assert err == ""


@pytest.mark.unit
def test_showNodes_default_fields_includes_favorite(_iface_with_favorite_nodes):
    """Test that isFavorite is included in default fields"""
    iface = _iface_with_favorite_nodes

    # Call showNodes which uses default fields
    result = iface.showNodes()

    # The result should contain the formatted table as a string
    assert "Fav" in result


@pytest.mark.unit
def test_showNodes_backward_compatibility_missing_field(capsys, _iface_with_favorite_nodes):
    """Test that nodes without isFavorite field are handled gracefully"""
    iface = _iface_with_favorite_nodes
    iface.showNodes()
    out, err = capsys.readouterr()

    # Find the legacy node line
    lines = out.split('\n')
    legacy_line = None
    for line in lines:
        if "Legacy Node" in line or "LEG1" in line:
            legacy_line = line
            break

    # Verify the legacy node appears in output
    assert legacy_line is not None, "Legacy node without isFavorite field should appear in output"

    # Verify it doesn't have an asterisk (should be treated as non-favorite)
    assert legacy_line.count("*") == 0, "Legacy node should not have asterisk (treated as non-favorite)"

    assert err == ""



"""TCPInterface class for interfacing with http endpoint
"""
# pylint: disable=R0917
import contextlib
import logging
import socket
import time
from typing import Optional

from meshtastic.stream_interface import StreamInterface

DEFAULT_TCP_PORT = 4403
logger = logging.getLogger(__name__)

class TCPInterface(StreamInterface):
    """Interface class for meshtastic devices over a TCP link"""

    def __init__(
        self,
        hostname: str,
        debugOut=None,
        noProto: bool=False,
        connectNow: bool=True,
        portNumber: int=DEFAULT_TCP_PORT,
        noNodes:bool=False,
        timeout: int = 300,
    ):
        """Constructor, opens a connection to a specified IP address/hostname

        Keyword Arguments:
            hostname {string} -- Hostname/IP address of the device to connect to
            timeout -- How long to wait for replies (default: 300 seconds)
        """

        self.stream = None

        self.hostname: str = hostname
        self.portNumber: int = portNumber

        self.socket: Optional[socket.socket] = None

        if connectNow:
            self.myConnect()
        else:
            self.socket = None

        super().__init__(debugOut=debugOut, noProto=noProto, connectNow=connectNow, noNodes=noNodes, timeout=timeout)

    def __repr__(self):
        rep = f"TCPInterface({self.hostname!r}"
        if self.debugOut is not None:
            rep += f", debugOut={self.debugOut!r}"
        if self.noProto:
            rep += ", noProto=True"
        if self.socket is None:
            rep += ", connectNow=False"
        if self.portNumber != DEFAULT_TCP_PORT:
            rep += f", portNumber={self.portNumber!r}"
        if self.noNodes:
            rep += ", noNodes=True"
        rep += ")"
        return rep

    def _socket_shutdown(self) -> None:
        """Shutdown the socket.
        Note: Broke out this line so the exception could be unit tested.
        """
        if self.socket is not None:
            self.socket.shutdown(socket.SHUT_RDWR)

    def myConnect(self) -> None:
        """Connect to socket"""
        logger.debug(f"Connecting to {self.hostname}") # type: ignore[str-bytes-safe]
        server_address = (self.hostname, self.portNumber)
        self.socket = socket.create_connection(server_address)

    def close(self) -> None:
        """Close a connection to the device"""
        logger.debug("Closing TCP stream")
        super().close()
        # Sometimes the socket read might be blocked in the reader thread.
        # Therefore we force the shutdown by closing the socket here
        self._wantExit = True
        if self.socket is not None:
            with contextlib.suppress(Exception):  # Ignore errors in shutdown, because we might have a race with the server
                self._socket_shutdown()
            self.socket.close()

        self.socket = None

    def _writeBytes(self, b: bytes) -> None:
        """Write an array of bytes to our stream and flush"""
        if self.socket is not None:
            self.socket.send(b)

    def _readBytes(self, length) -> Optional[bytes]:
        """Read an array of bytes from our stream"""
        if self.socket is not None:
            data = self.socket.recv(length)
            # empty byte indicates a disconnected socket,
            # we need to handle it to avoid an infinite loop reading from null socket
            if data == b'':
                logger.debug("dead socket, re-connecting")
                # cleanup and reconnect socket without breaking reader thread
                with contextlib.suppress(Exception):
                    self._socket_shutdown()
                self.socket.close()
                self.socket = None
                time.sleep(1)
                self.myConnect()
                self._startConfig()
                return None
            return data

        # no socket, break reader thread
        self._wantExit = True
        return None



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Custom options for defining:
- Maximum size of string/bytes
- Maximum number of elements in array

These are used by nanopb to generate statically allocable structures
for memory-limited environments.
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.descriptor_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.internal.extension_dict
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _FieldType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FieldTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FieldType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    FT_DEFAULT: _FieldType.ValueType  # 0
    """Automatically decide field type, generate static field if possible."""
    FT_CALLBACK: _FieldType.ValueType  # 1
    """Always generate a callback field."""
    FT_POINTER: _FieldType.ValueType  # 4
    """Always generate a dynamically allocated field."""
    FT_STATIC: _FieldType.ValueType  # 2
    """Generate a static field or raise an exception if not possible."""
    FT_IGNORE: _FieldType.ValueType  # 3
    """Ignore the field completely."""
    FT_INLINE: _FieldType.ValueType  # 5
    """Legacy option, use the separate 'fixed_length' option instead"""

class FieldType(_FieldType, metaclass=_FieldTypeEnumTypeWrapper): ...

FT_DEFAULT: FieldType.ValueType  # 0
"""Automatically decide field type, generate static field if possible."""
FT_CALLBACK: FieldType.ValueType  # 1
"""Always generate a callback field."""
FT_POINTER: FieldType.ValueType  # 4
"""Always generate a dynamically allocated field."""
FT_STATIC: FieldType.ValueType  # 2
"""Generate a static field or raise an exception if not possible."""
FT_IGNORE: FieldType.ValueType  # 3
"""Ignore the field completely."""
FT_INLINE: FieldType.ValueType  # 5
"""Legacy option, use the separate 'fixed_length' option instead"""
global___FieldType = FieldType

class _IntSize:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _IntSizeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_IntSize.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    IS_DEFAULT: _IntSize.ValueType  # 0
    """Default, 32/64bit based on type in .proto"""
    IS_8: _IntSize.ValueType  # 8
    IS_16: _IntSize.ValueType  # 16
    IS_32: _IntSize.ValueType  # 32
    IS_64: _IntSize.ValueType  # 64

class IntSize(_IntSize, metaclass=_IntSizeEnumTypeWrapper): ...

IS_DEFAULT: IntSize.ValueType  # 0
"""Default, 32/64bit based on type in .proto"""
IS_8: IntSize.ValueType  # 8
IS_16: IntSize.ValueType  # 16
IS_32: IntSize.ValueType  # 32
IS_64: IntSize.ValueType  # 64
global___IntSize = IntSize

class _TypenameMangling:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _TypenameManglingEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_TypenameMangling.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    M_NONE: _TypenameMangling.ValueType  # 0
    """Default, no typename mangling"""
    M_STRIP_PACKAGE: _TypenameMangling.ValueType  # 1
    """Strip current package name"""
    M_FLATTEN: _TypenameMangling.ValueType  # 2
    """Only use last path component"""
    M_PACKAGE_INITIALS: _TypenameMangling.ValueType  # 3
    """Replace the package name by the initials"""

class TypenameMangling(_TypenameMangling, metaclass=_TypenameManglingEnumTypeWrapper): ...

M_NONE: TypenameMangling.ValueType  # 0
"""Default, no typename mangling"""
M_STRIP_PACKAGE: TypenameMangling.ValueType  # 1
"""Strip current package name"""
M_FLATTEN: TypenameMangling.ValueType  # 2
"""Only use last path component"""
M_PACKAGE_INITIALS: TypenameMangling.ValueType  # 3
"""Replace the package name by the initials"""
global___TypenameMangling = TypenameMangling

class _DescriptorSize:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DescriptorSizeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DescriptorSize.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DS_AUTO: _DescriptorSize.ValueType  # 0
    """Select minimal size based on field type"""
    DS_1: _DescriptorSize.ValueType  # 1
    """1 word; up to 15 byte fields, no arrays"""
    DS_2: _DescriptorSize.ValueType  # 2
    """2 words; up to 4095 byte fields, 4095 entry arrays"""
    DS_4: _DescriptorSize.ValueType  # 4
    """4 words; up to 2^32-1 byte fields, 2^16-1 entry arrays"""
    DS_8: _DescriptorSize.ValueType  # 8
    """8 words; up to 2^32-1 entry arrays"""

class DescriptorSize(_DescriptorSize, metaclass=_DescriptorSizeEnumTypeWrapper): ...

DS_AUTO: DescriptorSize.ValueType  # 0
"""Select minimal size based on field type"""
DS_1: DescriptorSize.ValueType  # 1
"""1 word; up to 15 byte fields, no arrays"""
DS_2: DescriptorSize.ValueType  # 2
"""2 words; up to 4095 byte fields, 4095 entry arrays"""
DS_4: DescriptorSize.ValueType  # 4
"""4 words; up to 2^32-1 byte fields, 2^16-1 entry arrays"""
DS_8: DescriptorSize.ValueType  # 8
"""8 words; up to 2^32-1 entry arrays"""
global___DescriptorSize = DescriptorSize

@typing.final
class NanoPBOptions(google.protobuf.message.Message):
    """This is the inner options message, which basically defines options for
    a field. When it is used in message or file scope, it applies to all
    fields.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAX_SIZE_FIELD_NUMBER: builtins.int
    MAX_LENGTH_FIELD_NUMBER: builtins.int
    MAX_COUNT_FIELD_NUMBER: builtins.int
    INT_SIZE_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    LONG_NAMES_FIELD_NUMBER: builtins.int
    PACKED_STRUCT_FIELD_NUMBER: builtins.int
    PACKED_ENUM_FIELD_NUMBER: builtins.int
    SKIP_MESSAGE_FIELD_NUMBER: builtins.int
    NO_UNIONS_FIELD_NUMBER: builtins.int
    MSGID_FIELD_NUMBER: builtins.int
    ANONYMOUS_ONEOF_FIELD_NUMBER: builtins.int
    PROTO3_FIELD_NUMBER: builtins.int
    PROTO3_SINGULAR_MSGS_FIELD_NUMBER: builtins.int
    ENUM_TO_STRING_FIELD_NUMBER: builtins.int
    FIXED_LENGTH_FIELD_NUMBER: builtins.int
    FIXED_COUNT_FIELD_NUMBER: builtins.int
    SUBMSG_CALLBACK_FIELD_NUMBER: builtins.int
    MANGLE_NAMES_FIELD_NUMBER: builtins.int
    CALLBACK_DATATYPE_FIELD_NUMBER: builtins.int
    CALLBACK_FUNCTION_FIELD_NUMBER: builtins.int
    DESCRIPTORSIZE_FIELD_NUMBER: builtins.int
    DEFAULT_HAS_FIELD_NUMBER: builtins.int
    INCLUDE_FIELD_NUMBER: builtins.int
    EXCLUDE_FIELD_NUMBER: builtins.int
    PACKAGE_FIELD_NUMBER: builtins.int
    TYPE_OVERRIDE_FIELD_NUMBER: builtins.int
    SORT_BY_TAG_FIELD_NUMBER: builtins.int
    FALLBACK_TYPE_FIELD_NUMBER: builtins.int
    max_size: builtins.int
    """Allocated size for 'bytes' and 'string' fields.
    For string fields, this should include the space for null terminator.
    """
    max_length: builtins.int
    """Maximum length for 'string' fields. Setting this is equivalent
    to setting max_size to a value of length+1.
    """
    max_count: builtins.int
    """Allocated number of entries in arrays ('repeated' fields)"""
    int_size: global___IntSize.ValueType
    """Size of integer fields. Can save some memory if you don't need
    full 32 bits for the value.
    """
    type: global___FieldType.ValueType
    """Force type of field (callback or static allocation)"""
    long_names: builtins.bool
    """Use long names for enums, i.e. EnumName_EnumValue."""
    packed_struct: builtins.bool
    """Add 'packed' attribute to generated structs.
    Note: this cannot be used on CPUs that break on unaligned
    accesses to variables.
    """
    packed_enum: builtins.bool
    """Add 'packed' attribute to generated enums."""
    skip_message: builtins.bool
    """Skip this message"""
    no_unions: builtins.bool
    """Generate oneof fields as normal optional fields instead of union."""
    msgid: builtins.int
    """integer type tag for a message"""
    anonymous_oneof: builtins.bool
    """decode oneof as anonymous union"""
    proto3: builtins.bool
    """Proto3 singular field does not generate a "has_" flag"""
    proto3_singular_msgs: builtins.bool
    """Force proto3 messages to have no "has_" flag.
    This was default behavior until nanopb-0.4.0.
    """
    enum_to_string: builtins.bool
    """Generate an enum->string mapping function (can take up lots of space)."""
    fixed_length: builtins.bool
    """Generate bytes arrays with fixed length"""
    fixed_count: builtins.bool
    """Generate repeated field with fixed count"""
    submsg_callback: builtins.bool
    """Generate message-level callback that is called before decoding submessages.
    This can be used to set callback fields for submsgs inside oneofs.
    """
    mangle_names: global___TypenameMangling.ValueType
    """Shorten or remove package names from type names.
    This option applies only on the file level.
    """
    callback_datatype: builtins.str
    """Data type for storage associated with callback fields."""
    callback_function: builtins.str
    """Callback function used for encoding and decoding.
    Prior to nanopb-0.4.0, the callback was specified in per-field pb_callback_t
    structure. This is still supported, but does not work inside e.g. oneof or pointer
    fields. Instead, a new method allows specifying a per-message callback that
    will be called for all callback fields in a message type.
    """
    descriptorsize: global___DescriptorSize.ValueType
    """Select the size of field descriptors. This option has to be defined
    for the whole message, not per-field. Usually automatic selection is
    ok, but if it results in compilation errors you can increase the field
    size here.
    """
    default_has: builtins.bool
    """Set default value for has_ fields."""
    package: builtins.str
    """Package name that applies only for nanopb."""
    type_override: google.protobuf.descriptor_pb2.FieldDescriptorProto.Type.ValueType
    """Override type of the field in generated C code. Only to be used with related field types"""
    sort_by_tag: builtins.bool
    """Due to historical reasons, nanopb orders fields in structs by their tag number
    instead of the order in .proto. Set this to false to keep the .proto order.
    The default value will probably change to false in nanopb-0.5.0.
    """
    fallback_type: global___FieldType.ValueType
    """Set the FT_DEFAULT field conversion strategy.
    A field that can become a static member of a c struct (e.g. int, bool, etc)
    will be a a static field.
    Fields with dynamic length are converted to either a pointer or a callback.
    """
    @property
    def include(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Extra files to include in generated `.pb.h`"""

    @property
    def exclude(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Automatic includes to exclude from generated `.pb.h`
        Same as nanopb_generator.py command line flag -x.
        """

    def __init__(
        self,
        *,
        max_size: builtins.int | None = ...,
        max_length: builtins.int | None = ...,
        max_count: builtins.int | None = ...,
        int_size: global___IntSize.ValueType | None = ...,
        type: global___FieldType.ValueType | None = ...,
        long_names: builtins.bool | None = ...,
        packed_struct: builtins.bool | None = ...,
        packed_enum: builtins.bool | None = ...,
        skip_message: builtins.bool | None = ...,
        no_unions: builtins.bool | None = ...,
        msgid: builtins.int | None = ...,
        anonymous_oneof: builtins.bool | None = ...,
        proto3: builtins.bool | None = ...,
        proto3_singular_msgs: builtins.bool | None = ...,
        enum_to_string: builtins.bool | None = ...,
        fixed_length: builtins.bool | None = ...,
        fixed_count: builtins.bool | None = ...,
        submsg_callback: builtins.bool | None = ...,
        mangle_names: global___TypenameMangling.ValueType | None = ...,
        callback_datatype: builtins.str | None = ...,
        callback_function: builtins.str | None = ...,
        descriptorsize: global___DescriptorSize.ValueType | None = ...,
        default_has: builtins.bool | None = ...,
        include: collections.abc.Iterable[builtins.str] | None = ...,
        exclude: collections.abc.Iterable[builtins.str] | None = ...,
        package: builtins.str | None = ...,
        type_override: google.protobuf.descriptor_pb2.FieldDescriptorProto.Type.ValueType | None = ...,
        sort_by_tag: builtins.bool | None = ...,
        fallback_type: global___FieldType.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["anonymous_oneof", b"anonymous_oneof", "callback_datatype", b"callback_datatype", "callback_function", b"callback_function", "default_has", b"default_has", "descriptorsize", b"descriptorsize", "enum_to_string", b"enum_to_string", "fallback_type", b"fallback_type", "fixed_count", b"fixed_count", "fixed_length", b"fixed_length", "int_size", b"int_size", "long_names", b"long_names", "mangle_names", b"mangle_names", "max_count", b"max_count", "max_length", b"max_length", "max_size", b"max_size", "msgid", b"msgid", "no_unions", b"no_unions", "package", b"package", "packed_enum", b"packed_enum", "packed_struct", b"packed_struct", "proto3", b"proto3", "proto3_singular_msgs", b"proto3_singular_msgs", "skip_message", b"skip_message", "sort_by_tag", b"sort_by_tag", "submsg_callback", b"submsg_callback", "type", b"type", "type_override", b"type_override"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["anonymous_oneof", b"anonymous_oneof", "callback_datatype", b"callback_datatype", "callback_function", b"callback_function", "default_has", b"default_has", "descriptorsize", b"descriptorsize", "enum_to_string", b"enum_to_string", "exclude", b"exclude", "fallback_type", b"fallback_type", "fixed_count", b"fixed_count", "fixed_length", b"fixed_length", "include", b"include", "int_size", b"int_size", "long_names", b"long_names", "mangle_names", b"mangle_names", "max_count", b"max_count", "max_length", b"max_length", "max_size", b"max_size", "msgid", b"msgid", "no_unions", b"no_unions", "package", b"package", "packed_enum", b"packed_enum", "packed_struct", b"packed_struct", "proto3", b"proto3", "proto3_singular_msgs", b"proto3_singular_msgs", "skip_message", b"skip_message", "sort_by_tag", b"sort_by_tag", "submsg_callback", b"submsg_callback", "type", b"type", "type_override", b"type_override"]) -> None: ...

global___NanoPBOptions = NanoPBOptions

NANOPB_FILEOPT_FIELD_NUMBER: builtins.int
NANOPB_MSGOPT_FIELD_NUMBER: builtins.int
NANOPB_ENUMOPT_FIELD_NUMBER: builtins.int
NANOPB_FIELD_NUMBER: builtins.int
nanopb_fileopt: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, global___NanoPBOptions]
nanopb_msgopt: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, global___NanoPBOptions]
nanopb_enumopt: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.EnumOptions, global___NanoPBOptions]
nanopb: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, global___NanoPBOptions]



""" Serial interface class
"""
# pylint: disable=R0917
import logging
import sys
import time
from io import TextIOWrapper

from typing import List, Optional

import serial # type: ignore[import-untyped]

import meshtastic.util
from meshtastic.stream_interface import StreamInterface

logger = logging.getLogger(__name__)

class SerialInterface(StreamInterface):
    """Interface class for meshtastic devices over a serial link"""

    def __init__(
        self,
        devPath: Optional[str] = None,
        debugOut=None,
        noProto: bool = False,
        connectNow: bool = True,
        noNodes: bool = False,
        timeout: int = 300
    ) -> None:
        """Constructor, opens a connection to a specified serial port, or if unspecified try to
        find one Meshtastic device by probing

        Keyword Arguments:
            devPath {string} -- A filepath to a device, i.e. /dev/ttyUSB0 (default: {None})
            debugOut {stream} -- If a stream is provided, any debug serial output from the device will be emitted to that stream. (default: {None})
            timeout -- How long to wait for replies (default: 300 seconds)
        """
        self.noProto = noProto

        self.devPath: Optional[str] = devPath

        if self.devPath is None:
            ports: List[str] = meshtastic.util.findPorts(True)
            logger.debug(f"ports:{ports}")
            if len(ports) == 0:
                print("No Serial Meshtastic device detected, attempting TCP connection on localhost.")
                return
            elif len(ports) > 1:
                message: str = "Warning: Multiple serial ports were detected so one serial port must be specified with the '--port'.\n"
                message += f"  Ports detected:{ports}"
                meshtastic.util.our_exit(message)
            else:
                self.devPath = ports[0]

        logger.debug(f"Connecting to {self.devPath}")

        if sys.platform != "win32":
            with open(self.devPath, encoding="utf8") as f:
                self._set_hupcl_with_termios(f)
            time.sleep(0.1)

        self.stream = serial.Serial(
            self.devPath, 115200, exclusive=True, timeout=0.5, write_timeout=0
        )
        self.stream.flush()	# type: ignore[attr-defined]
        time.sleep(0.1)

        StreamInterface.__init__(
            self, debugOut=debugOut, noProto=noProto, connectNow=connectNow, noNodes=noNodes, timeout=timeout
        )

    def _set_hupcl_with_termios(self, f: TextIOWrapper):
        """first we need to set the HUPCL so the device will not reboot based on RTS and/or DTR
        see https://github.com/pyserial/pyserial/issues/124
        """
        if sys.platform == "win32":
            return

        import termios  # pylint: disable=C0415,E0401
        attrs = termios.tcgetattr(f)
        attrs[2] = attrs[2] & ~termios.HUPCL
        termios.tcsetattr(f, termios.TCSAFLUSH, attrs)

    def __repr__(self):
        rep = f"SerialInterface(devPath={self.devPath!r}"
        if hasattr(self, 'debugOut') and self.debugOut is not None:
            rep += f", debugOut={self.debugOut!r}"
        if self.noProto:
            rep += ", noProto=True"
        if hasattr(self, 'noNodes') and self.noNodes:
            rep += ", noNodes=True"
        rep += ")"
        return rep

    def close(self) -> None:
        """Close a connection to the device"""
        if self.stream:  # Stream can be null if we were already closed
            self.stream.flush()  # FIXME: why are there these  two flushes with 100ms sleeps?  This shouldn't be necessary
            time.sleep(0.1)
            self.stream.flush()
            time.sleep(0.1)
        logger.debug("Closing Serial stream")
        StreamInterface.close(self)
